<?php

use	Nette\Image,
	Nette\Caching\Cache,
	Nette\Caching\Storages\FileStorage;

/**
 * Recording helping methods
 */
class FeedRepository extends Nette\Object
{
	/** @var string $articlesRepository */
	private $articlesRepository;
	/** @var $feedOptions */
	public $feedOptions;
	/** @var string $phpQuery */
	private $phpQuery;
	/** @var FileStorage $cache */
	private $cache;
	/** @var Image Directory $imgDir */
	private $imageDir;
	/** @var www Dir $wwwDir */
	private $wwwDir;

	public function __construct($feedOptions,
								$imageDir,
								$wwwDir,
								PhpQuery $phpQuery,
								FileStorage $cache,
								ArticlesRepository $articlesRepository )
	{
		$this->feedOptions 			= $feedOptions;
		$this->imageDir 			= $imageDir;
		$this->wwwDir				= $wwwDir;
		$this->phpQuery    			= $phpQuery;
		$this->articlesRepository 	= $articlesRepository;
		$this->cache 	 			= new Cache($cache, 'htmlFront');
	}


	/**
	 * Get Articles from cache or RSS
	 *
	 * @return array Articles array
	 */
	public function getArticles()
	{
		$articles = $this->cache->load('rss');

		// Force to not use cached files (just for development) - LEAVE COMMENTED
		// $articles = NULL;

		if ($articles === NULL) {
			$json 		= $this->getRssJson();
			$articles 	= $this->buildArticles($json);
			$this->cache->save('rss', $articles, array(
			    Cache::EXPIRE => '+ 2 hours',
			));
		}
		return $articles;
	}

	/**
	 * Retrieve RSS Json string
	 * @return php articles array from JSON string
	 */
	private function getRssJson()
	{
		// supress the file_get_contents error witch "@""
		if ( $feed = @file_get_contents($this->feedOptions['url'])) {			
			$fileContents = str_replace(array("\n", "\r", "\t"), '', $feed);
			$fileContents = trim(str_replace('"', "'", $fileContents));
			$simpleXml = simplexml_load_string($fileContents);
			$json = json_encode($simpleXml);

			return $json;

		} else {
			throw new Exception('RSS feed is not reachable', 1);
		}
	}

	private function buildArticles($json)
	{

		$json = json_decode($json)->channel->item;
		$id = 0;

		foreach ($json as $article) {

			// Load content from rss
			$content = @file_get_contents($article->guid);

			$content = ForceUTF8\Encoding::toUTF8($content);

			$phpQueryObject = $this->phpQuery->newDocument( $content );

			// Load body images
			$image = pq($this->feedOptions['image']);
			$temp  = $image->attr('src');

			if (!empty($temp)){
				$image = $temp;
			} else {
				continue;
			}

			// Load teaser
			$temp = pq($this->feedOptions['teaser']);
			$temp  = $temp->html();
			if (!empty($temp)){
				$teaser = $temp;
			} else {
				continue;
			}

			// Load Body
			$temp = pq($this->feedOptions['body']);
			$temp  = $temp->html();
			if (!empty($temp)){
				$body = $temp;
				$in = mb_detect_encoding($body, mb_detect_order(), true);
			} else {
				continue;
			}

			if (!isset($article->category) || is_array($article->category)) {
				$article->category = 'category' . rand(1,9);
			}

			// Build rest of the RSS properties
			$articles[$id]['image']		= $image;
			$articles[$id]['teaser']	= $teaser;
			$articles[$id]['body']		= $body;
			$articles[$id]['link'] 		= $article->link;
			$articles[$id]['title'] 	= $article->title;
			$articles[$id]['category']	= $article->category;
			$articles[$id]['pubDate'] 	= gmdate('d.m.Y H:i', strtotime($article->pubDate));

			$id++;
		}

		return $articles;
	}

	/**
	 * Record Articles to DB
	 *
	 * @param  array  $articles
	 */
	public function recordArticles(array $articles)
	{

		if ( sizeof($articles) == 0 ) {
			throw new Execption('There is nothing to record. RSS Empty.',1);
		} else {

			// Filter out articles with images <= 520px width
			$articles = $this->filterByImageSize($articles, 300);
			// Filter out articles already in DB
			$lastInDB = $this->articlesRepository->oldestArticleInDB();

			if ($lastInDB === FALSE) {
				$lastIdInDB = 0;
				$articles 	= $this->arrayByDateAsc($articles);
			} else {
				// Set the proper counter for new articles in DB
				$articles 	= $this->filterByDate($articles, $lastInDB->pub_date);
				$lastIdInDB = $lastInDB->id;
			}

			foreach ($articles as $newArticle) {

				$lastIdInDB++;
				$newArticle 		= $this->saveImages($newArticle, $lastIdInDB);
				$newArticle['body'] = $this->externalLinks($newArticle['body']);
				$this->articlesRepository->save($newArticle);
			}
		}

	}

	/**
	 * Filter out articles with insufficiently wide images
	 * @param array new articles
	 * @param var imageSize
	 * @return array articles with proper images
	 */
	public function filterByImageSize($articles, $imgFilterWidth)
	{

		$filteredArticles = array();

		foreach ($articles as $article) {

			$image = Image::fromFile($article['image']);

			if ($image->width >= $imgFilterWidth ){
				$filteredArticles[] = $article;
			}

		}
		return $filteredArticles;
	}

	/**
	 * Filter out articles wich already are in DB
	 * @param array php variable with new articles
	 * @return array articles for recording in DB
	 */
	public function filterByDate($articles, $date = 0)
	{
		$filteredArticles 	= array();
		$articles 			= $this->arrayByDateAsc($articles);

		foreach ($articles as $article) {

			$artDate = strtotime($article['pubDate']);
			$dbDate  = strtotime($date);
			if ( $artDate > $dbDate ){
				$filteredArticles[] = $article;
			}
		}
		return $filteredArticles;
	}

	/**
	 * Helper function for array sorting
	 * @param  array $a array A
	 * @param  array $b array B
	 * @return int      difference
	 */
	private function sortFunction($a, $b)
	{
		return strtotime($a['pubDate']) - strtotime($b['pubDate']) ;
	}

	/**
	 * Sort Articles by publication date Ascending
	 *
	 * @param  array  $articles Input Array
	 * @return array            Sorted Array
	 */
	public function arrayByDateAsc(array $articles)
	{
		usort($articles, array($this, 'sortFunction'));
		return $articles;
	}

	/**
	 * Treat external links
	 * @param html code
	 * @return html code without external links
	 */
	public function externalLinks($html)
	{
		$html = $this->phpQuery->newDocument($html);
		pq('a')->attr("href", "#")->attr("target", "")->attr("class", "external");
		return $html->html();
	}

	/**
	 * Save images to local folder
	 *
	 * @param array $art Article
	 * @param int   $id ID
	 * @return array Article
	 */
	public function saveImages($art, $id)
	{
		/** Handle big title image */

		$imgFolder = $this->wwwDir . '/' . $this->imageDir;

		if(!is_dir($imgFolder . '/' . $id))
		{
			// Create new image directory for current article
			mkdir($imgFolder . '/' . $id);
		}

		$titleImage = Image::fromFile($art['image']);

		// Title image gets new name "title.jpg"
		$newName = $id . "/title.jpg";
		$titleImage->save( $imgFolder . '/' . $newName, 80, Image::JPEG);

		// Change original url to the local url
		$art['image'] = $newName;

		/** Handle smaller body images */

		// Load body images
		// $bodyImages = $this->phpQuery->newDocument($art->body);
		$bodyImages = $this->phpQuery->newDocument($art['body']);
		$images = pq('img');

		// Save body images to the local folder and change src urls
		$imageID = 0;
		foreach ($images as $image)
		{
			$imageID++;
			$imageSelector = pq($image);
			$image = Image::fromFile($imageSelector->attr('src'));

			// Save Image with a new name
			$smallImageName = $id . "/" . $imageID . ".jpg";
			$image->save( $imgFolder . '/' . $smallImageName, 80, Image::JPEG);

			// Change src attributes to local
			$imageSelector->attr('src', $smallImageName);
		}
		$art['body'] = $bodyImages->html();

		return $art;
	}


	// /**
	//  * Filter out articles with insufficiently wide images
	//  * @param array new articles
	//  * @param var imageSize
	//  * @return array articles with proper images
	//  */
	// public function filterByImageSize($articles, $imgFilterWidth)
	// {

	// 	$filteredArticles = array();

	// 	foreach ($articles as $article) {

	// 		if ( $article->teaser && $article->body ) {

	// 			if (is_array($article->imgs)){
	// 				if ($article->imgs[0]){
	// 					if ($article->imgs[0]->width >= $imgFilterWidth ){

	// 						$filteredArticles[] = $article;
	// 					}
	// 				}
	// 			} else {
	// 				if ($article->imgs){
	// 					if ($article->imgs->width >= $imgFilterWidth ){

	// 						$filteredArticles[] = $article;
	// 					}
	// 				}
	// 			}
	// 		}

	// 	}
	// 	return $filteredArticles;
	// }
}
