<?php
require '../libs/Nette/loader.php';
require('../libs/phpQuery/phpQuery/phpQuery.php');

use Nette\Diagnostics\Debugger,
	Nette\Utils\Finder,
	Nette\Image;

Debugger::enable(); // enable Nette Debugger

/**
 * Define basic PHP constants - e.g. local image storage
 */
define("IMAGES_DIR", "/Users/stic/Sites/kepler_news/www/images/article_images/");
define("IMAGES_URL", "http://localhost:8888/kepler_news/www/images/article_images/");

/**
 * Url address of Yahoo Pipes RSS Feed
 * @var $rssFeed varchar
 */
$rssFeed = 'http://pipes.yahoo.com/pipes/pipe.run?RssUrl=http%3A%2F%2Fwww.spiegel.de%2Fschlagzeilen%2Findex.rss&_id=e6c6b249a3b614aadc896f2aafbf458f&_render=json';

/**
 * Database Conenction Credentials
 */
$mysql = array(
		'dsn' 		=> 'mysql:unix_socket=/Applications/MAMP/tmp/mysql/mysql.sock;dbname=kepler_news',
		'user' 		=> 'root',
		'password' 	=> 'root'
);

/**
 * DI Container for database connection and articles factory
 */
class DIContainer
{
	/** @var array $parameters - database connection parameters */
	private $parameters;
	/** @var string $rssFeed */
	private $rssFeed;
	/** @array Nette\Database\Connection */
	private $services = array();

	public function __construct(array $parameters, $rssFeed){

		$this->parameters = $parameters;
		$this->rssFeed = $rssFeed;
	}

	public function createConnection()
	{
		return new Nette\Database\Connection(
			$this->parameters['dsn'],
			$this->parameters['user'],
			$this->parameters['password']
		);
	}

	public function getConnection()
	{
		if(!isset($this->services['connection'])) {
			$this->services['connection'] = $this->createConnection();
		}
		return $this->services['connection'];
	}

	/**
	 * Factory for Article class
	 */
	public function createArticle()
	{

		return new Article($this->getConnection());
	}

	/**
	 * Retrieve RSS Json string
	 * @return php articles array from JSON string
	 */
	public function getRssJson()
	{
		$jsonResult = json_decode(file_get_contents($this->rssFeed));
		return $jsonResult->value->items;
	}

	/**
	 * Filter out articles with insufficiently wide images
	 * @param array php variable with new articles
	 * @return array articles with proper images
	 */
	public function filterByImageSize($articles, $imgFilterWidth)
	{

		$filteredArticles = array();

		foreach ($articles as $article) {
			if (is_array($article->imgs)){
				if ($article->imgs[0]){
					if ($article->imgs[0]->width >= $imgFilterWidth ){

						$filteredArticles[] = $article;
					}
				}
			} else {
				if ($article->imgs){
					if ($article->imgs->width >= $imgFilterWidth ){

						$filteredArticles[] = $article;
					}
				}
			}

		}
		return $filteredArticles;
	}

	/**
	 * Filter out articles wich already are in DB
	 * @param array php variable with new articles
	 * @return array articles for recording in DB
	 */
	public function filterByDate($articles, $date)
	{

		$filteredArticles = array();

		foreach ($articles as $article) {

			$new_date = new DateTime($article->pubDate);
			if ($new_date > $date ){

				// Debugger::Dump($new_date);
				$filteredArticles[] = $article;
			}
		}
		return $filteredArticles;
	}

	/**
	 * Select latest article from DB
	 */
	public function oldestArticleInDB(){
		return $oldest = $this->getConnection()->table('articles')->order('id DESC')->limit(1)->fetch();
	}

}

/**
 * Class for working with article and storing it to database
 */
class Article
{
	/** @var array content */
	public $content;
	/** @var Nette\Database\Connection connection */
	public $connection;


	public function __construct(Nette\Database\Connection $connection)
	{
		$this->connection = $connection;
	}

	/**
	 * Treat external links
	 * @param html code
	 * @return html code without external links
	 */
	public function externalLinks($html)
	{
		$html = phpQuery::newDocument($html);
		pq('a')->attr("href", "#")->attr("target", "")->attr("class", "external");
		return $html->html();
	}

	/**
	 * Save images to local folder
	 */
	public function saveImages($id)
	{


		/** Handle big title image */

		if(!is_dir(IMAGES_DIR . $id))
		{
				// Create new image directory for current article
			mkdir(IMAGES_DIR . $id);

			$src = is_array($this->content->imgs) ? $this->content->imgs[0]->src : $this->content->imgs->src;
			$titleImage = Image::fromFile($src);

				// Title image gets new name "title.jpg"
			$newName = $id . "/title.jpg";
			$titleImage->save( IMAGES_DIR . $newName, 80, Image::JPEG);

				// Change original url to the local url
			is_array($this->content->imgs) ? $this->content->imgs[0]->src = $newName : $this->content->imgs->src = $newName;



		/** Handle smaller body images */

				// Load body images
			$bodyImages = phpQuery::newDocument($this->content->body);
			$images = pq('img');

				// Save body images to the local folder and change src urls
			$imageID = 0;
			foreach ($images as $image)
			{
				$imageID++;
				$imageSelector = pq($image);
				$image = Image::fromFile($imageSelector->attr('src'));

					// Save Image with a new name
				$smallImageName = $id . "/" . $imageID . ".jpg";
				$image->save( IMAGES_DIR . $smallImageName, 80, Image::JPEG);

					// Change src attributes to local
				$imageSelector->attr('src', $smallImageName);
			}
			$this->content->body = $bodyImages->html();
		}
	}

	/**
	 * Save article to the database
	 */
	public function save()
	{
		$src = is_array($this->content->imgs) ? $this->content->imgs[0]->src : $this->content->imgs->src;

		return $this->connection->table('articles')->insert(array(
			'guid'			=> $this->content->guid,
			'title'			=> $this->content->title,
			'link'			=> $this->content->link,
			'category'		=> $this->content->category,
			'description'	=> $this->content->description,
			'images'		=> $src,
			'teaser'		=> $this->content->teaser,
			'body'			=> $this->content->body,
			'pub_date'		=> date_format(new DateTime($this->content->pubDate), 'Y-m-d H:i:s')
		));
	}

}

// Create DI container
$container = new DIContainer($mysql, $rssFeed);

// Retrieve the RSS feed as a PHP variable
$articles = $container->getRssJson();
Debugger::dump($articles);

// Filter out articles with images <= 520px width
$articles = $container->filterByImageSize($articles, 520);

// Filter out articles already in DB
$lastInDB = $container->oldestArticleInDB();
$articles = $container->filterByDate($articles, $lastInDB->pub_date);

// Set the proper counter for new articles in DB
$lastIdInDB = $lastInDB->id ? $lastInDB->id : 0 ;

// Debugger::dump($articles);
// Debugger::dump($lastIdInDB);

if ( sizeof($articles) == 0 ) {
	echo "Nothing to record";
}

foreach ($articles as $newArticle) {

	$lastIdInDB++;

	$art 			= $container->createArticle();
	$art->content 	= $newArticle;

	//$art->saveImages($lastIdInDB);
	//$art->content->body = $art->externalLinks($art->content->body);
	//$art->save();

	echo "Article id#" . $lastIdInDB . " Sucessfully recorded! <br />";

}
